%% Modèle de Référence - Simulation d'un Écoulement en Milieu Poreux
% Ce script génère un modèle de référence pour simuler un écoulement en milieu poreux biphasique (eau-huile).
% Le modèle est basé sur un champ aléatoire gaussien pour la perméabilité,
% et utilise des puits d'injection et de production pour observer le transport
% de fluides dans un réservoir hétérogène. Les modules de visualisation sont
% utilisés pour afficher les propriétés de chaque cellule. L'objectif est
% de déterminer les courbes de niveau d'eau (water cut) montrant le taux de
% récupération du pétrole en fonction du temps.

% Il s'agit d'un exemple d'initiation proposer dans le logiciel MRST
% (Matlab Reservoir Simulation Toolbox)

%% Génération d'un modèle géologique d'un réservoir
% On assume qu'il y a une seule unité.
% On assume une porosité constante (jamais le cas)
% Les dimensions du modèle sont de 250 m par 250m par 5m discrétisé en 51
% blocs en x par 51 bloc en y et 1 bloc en z.
grdecl = makeModel([51, 51, 1], [250, 250, 5]);
idxActiveCells = logical(grdecl.ACTNUM);

% Génération de la grille
G = processGRDECL(grdecl);
G = computeGeometry(G);

%% Champ de Perméabilité - Modèle Aléatoire Gaussien simuler par FFTMA
perm = fftma([4, 10, 10, 10, 0, 0, 0], 0.25, 45200, 1, 51, 1, 51, 1, 1, 1);
Zref = perm;

%% Propriétés de la Roche et Fluide
mrstModule add incomp;                         % Ajouter le module incomp pour l'écoulement
gravity on;                                    % Activer la gravité
rock = makeRock(G, 400 * milli * darcy, 0.20); % Propriétés uniformes de la roche

% Perméabilité anisotrope de la conductivité hydraulique (Tenseur K)
rock.perm = [10.^(perm(idxActiveCells) - 13), ...   % Kxx
             10.^(perm(idxActiveCells) - 13), ...   % Kyy
             10.^(perm(idxActiveCells) - 13) / 10]; % Kzz
clear perm;

% Définition d'un fluide simple. Le premier fluide est l'eau et le deuxième
% de l'huile
fluid = initSimpleFluid( ...
    'mu', [1, 5] * centi * poise, ...            % Viscosité
    'rho', [1014, 700] * kilogram / meter^3, ... % Densité
    'n', [2, 2]);                                % Lois de Corey

%% Configuration des Puits
% Il y a un puit injecteur (injecte de l'eau) et trois puits de production
% qui récubère l'huile

Q = 1800 * meter^3 / day;                      % Débit d'injection
W = verticalWell([], G, rock, 45, 35, 1, ...   % Puits d'injection (45, 35, 1)
    'Type', 'rate', 'Val', Q, ...
    'Radius', 0.1, 'InnerProduct', 'ip_tpf', ...
    'Comp_i', [1, 0], 'Name', 'I');

W = verticalWell(W, G, rock, 25, 10, 1, ...    % Puits de production P1 (25, 10, 1)
    'Type', 'rate', 'Val', -Q/3, ...
    'Radius', 0.1, 'InnerProduct', 'ip_tpf', ...
    'Comp_i', [0, 1], 'Name', 'P1');

W = verticalWell(W, G, rock, 6, 40, 1, ...     % Puits de production P2 (6, 40, 1)
    'Type', 'rate', 'Val', -Q/3, ...
    'Radius', 0.1, 'InnerProduct', 'ip_tpf', ...
    'Comp_i', [0, 1], 'Name', 'P2');

W = verticalWell(W, G, rock, 6, 21, 1, ...     % Puits de production P3 (6, 21, 1)
    'Type', 'rate', 'Val', -Q/3, ...
    'Radius', 0.1, 'InnerProduct', 'ip_tpf', ...
    'Comp_i', [0, 1], 'Name', 'P3');

%% Conditions Initiales et Résolution de l'Écoulement
idxs0 = G.cells.centroids(:, 3) > 5;
s0 = zeros(G.cells.num, 2);
s0(idxs0, 1) = 1;                               % Initialisation: saturé en huile
s0(~idxs0, 2) = 1;

sol = initState(G, [], 0, s0);

T = computeTrans(G, rock);                      % Transmissibilité
psolve = @(state) incompTPFA(state, G, T, fluid, 'wells', W);                   % Solveur pression
tsolve = @(state, dT) implicitTransport(state, G, dT, rock, fluid, 'wells', W); % Solveur transport
sol = psolve(sol);

%% Simulation du Transport entre les Puits
num_steps = 10;
time_steps = logspace(0, 3, num_steps) * day;  % Pas de temps logarithmique
solutions = cell(num_steps, 1);
WCref = zeros(num_steps, 3);                    
Widx = [129, 1553, 608];                        % Indice de la localisation des puits
for i = 1:num_steps
    sol = tsolve(sol, time_steps(i));
    sol = psolve(sol);
    solutions{i} = sol;
    WCref(i, :) = sol.s([129, 1553, 608]);
end

%% Visualisation des Données de Simulation
figure(1);
clf;
plotGrid(G, 'FaceAlpha', 0, 'EdgeAlpha', .1);
plotCellData(G, log10(rock.perm(:,1)) + 7);
plotWell(G, W(2:end), 'height', 2, 'color', 'k', 'color2', 'k'); view(30,50);
plotWell(G, W(1), 'height', 10, 'color', 'k', 'color2', 'k'); view(30,50);
colorbar; view(130,50);

figure(2);
clf;
plotGrid(G, 'FaceAlpha', 0, 'EdgeAlpha', .1);
plotCellData(G, sol.s(:,1));
plotWell(G, W, 'height', 2, 'color', 'k', 'color2', 'k'); view(30,50);
colorbar; view(130,50);

%% Tracé des Courbes de Niveau d'Eau avec Échelle Logarithmique pour le Temps
figure(3);
plot(cumsum(time_steps) / day, WCref(:, 1), '-o', 'DisplayName', 'Puits 1');
hold on;
plot(cumsum(time_steps) / day, WCref(:, 2), '-s', 'DisplayName', 'Puits 2');
plot(cumsum(time_steps) / day, WCref(:, 3), '-^', 'DisplayName', 'Puits 3');
hold off;

set(gca, 'XScale', 'log');  % Échelle logarithmique pour l'axe x
xlabel('Temps (jours)');
ylabel('Saturation en eau');
legend;
title('Évolution de la saturation en eau aux différents puits');
grid on;

%% Sauvegarder l'état initial du modèle
sol = initState(G, [], 0, s0);
save modelInit.mat G rock fluid W sol Widx WCref idxActiveCells Zref
